/*
 * Access a server offering a nested counter.
 *
 * Copyright (C) 2020, 2022 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>

#include <stdio.h>

#include "nested_counter_client.h"

int main(void)
{
  l4_cap_idx_t server = l4re_env_get_cap("server");

  /* Reference to a counter and encapsulation. */

  ref_NestedCounter counter_ref = {.cap=server};
  NestedCounter counter_obj = {.ref=counter_ref, .iface=&client_iface_NestedCounter};

  /* Reference to another counter. */

  ref_NestedCounter next_counter_ref = {.cap=L4_INVALID_CAP};
  NestedCounter next_counter_obj;

  /* Call the calculator. */

  int result;

  /* Call the counter. */

  /* Invoke the operation function directly. */

  NestedCounter_increment(counter_ref, &result);
  printf("increment -> %d\n", result);

  /* Invoke the operation via the interface. */

  counter_obj.iface->increment(counter_obj.ref, &result);
  printf("increment -> %d\n", result);

  /* Make a new counter. */

  counter_obj.iface->next(counter_obj.ref, &next_counter_ref.cap);
  printf("new counter: %lx\n", next_counter_ref.cap);
  next_counter_obj = (NestedCounter) {.ref=next_counter_ref, .iface=&client_iface_NestedCounter};

  /* Increment the next counter. */

  next_counter_obj.iface->increment(next_counter_obj.ref, &result);
  printf("increment (next) -> %d\n", result);

  /* Increment the original counter. */

  counter_obj.iface->increment(counter_obj.ref, &result);
  printf("increment -> %d\n", result);

  return 0;
}
